"""google sheets deferrals settings """
from mitol.common.envs import get_int, get_string

MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID = get_string(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID",
    description=(
        "ID of the worksheet within the enrollment change request spreadsheet that contains "
        "enrollment deferral requests"
    ),
)
MITOL_GOOGLE_SHEETS_DEFERRALS_FIRST_ROW = get_int(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_FIRST_ROW",
    default=5,
    description=(
        "The first row (as it appears in the spreadsheet) of data that our scripts should consider "
        "processing in the deferral request spreadsheet"
    ),
)

MITOL_GOOGLE_SHEETS_DEFERRALS_PLUGINS = get_string(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_PLUGINS",
    description="The path to your deferral plugin, example: app.plugins.DeferralPlugin",
)
MITOL_GOOGLE_SHEETS_DEFERRALS_PROCESSOR_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_PROCESSOR_COL",
    default=7,
    description=(
        "The zero-based index of the enrollment change sheet column that contains the user that processed the row"
    ),
)
MITOL_GOOGLE_SHEETS_DEFERRALS_COMPLETED_DATE_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_COMPLETED_DATE_COL",
    default=8,
    description=(
        "The zero-based index of the enrollment change sheet column that contains the row completion date"
    ),
)
MITOL_GOOGLE_SHEETS_DEFERRALS_ERROR_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_ERROR_COL",
    default=9,
    description=(
        "The zero-based index of the enrollment change sheet column that contains row processing error messages"
    ),
)
MITOL_GOOGLE_SHEETS_DEFERRALS_SKIP_ROW_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_DEFERRALS_SKIP_ROW_COL",
    default=10,
    description=(
        "The zero-based index of the enrollment change sheet column that indicates whether the row should be skipped"
    ),
)



""" mitol.google_sheets_deferrals """
default_app_config = "mitol.google_sheets_deferrals.apps.GoogleSheetsDeferralsApp"

__version__ = "2023.6.9"
__distributionname__ = "mitol-django-google-sheets-deferrals"



"""Google Sheets Deferrals app constants"""

SHEET_TYPE_ENROLL_CHANGE = "enrollchange"
WORKSHEET_TYPE_DEFERRAL = "deferral"

REQUIRED_GOOGLE_SHEETS_DEFERRALS_SETTINGS = [
    "MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID",
    "MITOL_GOOGLE_SHEETS_DEFERRALS_PLUGINS",
]



"""Admin for DeferralRequest"""
from django.contrib import admin

from mitol.google_sheets_deferrals.models import DeferralRequest


class DeferralRequestAdmin(admin.ModelAdmin):
    """Admin for DeferralRequest"""

    model = DeferralRequest
    list_display = ("form_response_id", "date_completed", "raw_data")


admin.site.register(DeferralRequest, DeferralRequestAdmin)




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/mitol/google_sheets_deferrals:mitol-django-google-sheets-deferrals

from setuptools import setup

setup(**{
    'authors': [
        'MIT Office of Open Learning <mitx-devops@mit.edu>',
    ],
    'classifiers': [
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    'description': 'Library to handle Deferral requests using Google Sheets integrations in Django',
    'install_requires': (
        'django<4.0,>=2.2.12',
        'mitol-django-common',
        'mitol-django-google-sheets',
        'pluggy==0.13.1',
        'setuptools',
    ),
    'license': 'BSD 3-Clause License',
    'long_description': """mitol-django-google-sheets-deferrals
---

This is the Open Learning Django Google Sheets library for handling deferral requests over Google Sheets
### Setup
Before you begin setting up this library make sure you set up `mitol-django-google-sheets` first. Follow the instructions [here](https://github.com/mitodl/ol-django/blob/85bea3ec5da01180ef943deb89b14d1463eb7c21/src/mitol/google_sheets/README.md).

Once you have `mitol-django-google-sheets` installed, then:

`pip install mitol-django-google-sheets-deferrals`

#### In `settings.py`

```python
INSTALLED_APPS = [
    ...
    "mitol.google_sheets_deferrals.apps.GoogleSheetsDeferralsApp",
]
```


```python
MITOL_GOOGLE_SHEETS_DEFERRALS_PLUGINS = [
    "ecommerce.plugins.DeferralPlugin"
]
```

#### In `ecommerce/plugins.py`
```python
from mitol.google_sheets_deferrals.hooks import hookimpl

class DeferralPlugin:
    @hookimpl
    def deferrals_process_request(deferral_request: DeferralRequestRow) -> DeferralResult:
        # TODO: look up the user/enrollment
        # TODO: unenroll user from and enroll to
```

#### Create your google sheets folder
The structure of the folder should look as the following:

1. Spreadsheet that contains sheets with 'Deferral Form Responses' and the 'Deferrals' sheets, that is being processed.
2. Request Form

Ask a developer for a template folder to copy this structure.


#### Add settings to your app

Set `MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID` in your .env file. It is required.

Some settings have default values, which you might need to update, if your sheet is different.

Description of settings (in addition to the ones described in `mitol-django-google-sheets`):

- `MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID`
  > The IDs of the refund and deferral sheets in the change of enrollment spreadsheet. These can
    be found by opening the spreadsheet, selecting the "Refunds" or "Deferrals" worksheets, and
    copying down the `gid` value in the URL.
    
    Example:
    `https://docs.google.com/spreadsheets/d/<MITOL_GOOGLE_SHEETS_ENROLLMENT_CHANGE_SHEET_ID>/edit#gid=<MITOL_GOOGLE_SHEETS_DEFERRALS_REQUEST_WORKSHEET_ID>`


- `MITOL_GOOGLE_SHEETS_DEFERRALS_PROCESSOR_COL` 
  > The zero-based index of the deferral change sheet column that contains the user that processed the row

- `MITOL_GOOGLE_SHEETS_DEFERRALS_COMPLETED_DATE_COL`
  > The zero-based index of the deferral change sheet column that contains the row completion date

- `MITOL_GOOGLE_SHEETS_DEFERRALS_ERROR_COL`
  > The zero-based index of the deferral change sheet column that contains row processing error messages

- `MITOL_GOOGLE_SHEETS_DEFERRALS_SKIP_ROW_COL`
  > The zero-based index of the deferral change sheet column that indicates whether the row should be skipped

- `MITOL_GOOGLE_SHEETS_DEFERRALS_FIRST_ROW`
  > The first row (as it appears in the spreadsheet) of data that our scripts should consider processing in the deferral request spreadsheet


### Usage
To processing your google sheet with deferral requests simply add:
```python
deferral_request_handler = DeferralRequestHandler()
results = deferral_request_handler.process_sheet()
```


Here are two main ways that we are currently getting our Google Sheets Deferrals processed:

1. Run a management command. For example:
`./manage.py process_deferral_requests -i "<spreadsheet id>"`
2. Set up a scheduled celery task. For example:
```python
from mitol.google_sheets_deferrals.api import DeferralRequestHandler
from main.celery import app

@app.task
def process_deferral_requests():
    \"\"\"
    Task to process deferral requests from Google sheets
    \"\"\"
    deferral_request_handler = DeferralRequestHandler()
    if not deferral_request_handler.is_configured():
        log.warning("MITOL_GOOGLE_SHEETS_* are not set")
        return
    deferral_request_handler.process_sheet()
```

### Develper setup
For local development setup and testing please follow the instructions `Developer Setup` [here](https://github.com/mitodl/ol-django/blob/85bea3ec5da01180ef943deb89b14d1463eb7c21/src/mitol/google_sheets/README.md#developer-setup).""",
    'long_description_content_type': 'text/markdown',
    'name': 'mitol-django-google-sheets-deferrals',
    'namespace_packages': (
        'mitol',
    ),
    'package_data': {
        'mitol.google_sheets_deferrals': (
            'CHANGELOG.md',
            'README.md',
            'py.typed',
        ),
    },
    'packages': (
        'mitol',
        'mitol.google_sheets_deferrals',
        'mitol.google_sheets_deferrals.management.commands',
        'mitol.google_sheets_deferrals.migrations',
        'mitol.google_sheets_deferrals.settings',
    ),
    'python_requires': '>=3.8',
    'version': '2023.6.9',
    'zip_safe': True,
})



"""Models for Deferrals"""
from mitol.google_sheets.models import GoogleSheetsRequestModel


class DeferralRequest(GoogleSheetsRequestModel):
    """Model that represents a request to defer an enrollment"""

    def __str__(self):
        return "DeferralRequest: id={}, form_response_id={}, completed={}".format(
            self.id, self.form_response_id, self.date_completed is not None
        )
